package com.jjt.biz.service.impl;

import com.alibaba.fastjson2.JSON;
import com.jjt.biz.service.IApiYrService;
import com.jjt.biz.vo.*;
import com.jjt.calc.domain.TwinCalcDayYhj;
import com.jjt.calc.domain.TwinCalcHourYhj;
import com.jjt.calc.service.ITwinCalcDayYhjService;
import com.jjt.common.constant.CacheConstants;
import com.jjt.common.core.redis.RedisCache;
import com.jjt.common.utils.DateUtils;
import com.jjt.rz.domain.TwinCalcHourRz;
import com.jjt.rz.domain.TwinDeviceTypeData;
import com.jjt.rz.service.ITwinCalcHourRzService;
import com.jjt.rz.service.ITwinDeviceRzService;
import com.jjt.rz.service.ITwinDeviceTypeService;
import com.jjt.rz.vo.CompareVo;
import com.jjt.ws.domain.TwinRzCalcMonth;
import com.jjt.ws.service.ITwinRzCalcMonthService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 首页统计数据
 *
 * @author wukai
 * @date 2024/5/4 20:35
 */
@Service
@Slf4j
public class ApiYrServiceImpl implements IApiYrService {
    @Resource
    private RedisCache redisCache;
    @Resource
    private ITwinRzCalcMonthService rzCalcMonthService;
    @Resource
    private ITwinCalcDayYhjService calcDayYhjService;
    @Resource
    private ITwinDeviceRzService rzService;
    @Resource
    private ITwinCalcHourRzService hourRzService;
    @Resource
    private ITwinDeviceTypeService typeService;

    /**
     * 获取产线状态
     *
     * @return 结果
     */
    @Override
    public YrProdLineStatusVO status() {
        YrProdLineStatusVO vo = new YrProdLineStatusVO();
        vo.mock();
        return vo;
    }

    /**
     * 获取生产效率
     *
     * @return 结果
     */
    @Override
    public List<YrProdEfficiencyVO> eff() {
        List<YrProdEfficiencyVO> list = new ArrayList<>();
        YrProdEfficiencyVO[] arr = new YrProdEfficiencyVO[9];
        for (int i = 0; i < 9; i++) {
            YrProdEfficiencyVO vo = new YrProdEfficiencyVO(i);
            vo.mock();
            arr[i] = vo;
        }
        YrProdEfficiencyVO line0 = new YrProdEfficiencyVO(0);
//        for (int i = 1; i < 9; i++) {
//            YrProdEfficiencyVO vo = arr[i];
//            line0.setEffA(line0.getEffA().add(vo.getEffA()));
//            line0.setEffB(line0.getEffB().add(vo.getEffB()));
//            line0.setTimeA(line0.getTimeA().add(vo.getTimeA()));
//            line0.setTimeB(line0.getTimeB().add(vo.getTimeB()));
//            line0.setLengthA(line0.getLengthA().add(vo.getLengthA()));
//            line0.setLengthB(line0.getLengthB().add(vo.getLengthB()));
//            line0.setWeightA(line0.getWeightA().add(vo.getWeightA()));
//            line0.setWeightB(line0.getWeightB().add(vo.getWeightB()));
//            line0.setWeight(line0.getWeight().add(vo.getWeight()));
//            line0.setLength(line0.getLength().add(vo.getLength()));
//
//        }

        List<TwinCalcHourYhj> calcList = redisCache.getCacheObject(CacheConstants.YHJ_TODAY);
        Map<Long, Map<String, Integer>> sumByDeviceAndTeam = calcList.stream()
                .collect(Collectors.groupingBy(
                        TwinCalcHourYhj::getDeviceId,
                        Collectors.groupingBy(
                                TwinCalcHourYhj::getTeam,
                                Collectors.summingInt(TwinCalcHourYhj::getLength)
                        )
                ));
        AtomicInteger openTimes = new AtomicInteger(0);
        sumByDeviceAndTeam.forEach((deviceId, teamMap) -> {
            Integer lengthA = teamMap.get("A");
            Integer lengthB = teamMap.get("B");
            if (lengthB == null) {
                lengthB = 0;
            }
            Integer length = lengthA + lengthB;
            YrProdEfficiencyVO vo = arr[deviceId.intValue()];
            if (length > 0) {
                openTimes.incrementAndGet();
            } else {
                vo.setOpen(0);
                vo.setRatio(BigDecimal.ZERO);
                vo.setTimeA(BigDecimal.ZERO);
                vo.setTimeB(BigDecimal.ZERO);
                vo.setEffA(BigDecimal.ZERO);
                vo.setEffB(BigDecimal.ZERO);
            }
            BigDecimal weightA = BigDecimal.valueOf(lengthA).divide(BigDecimal.valueOf(1000), 2, RoundingMode.HALF_UP);
            BigDecimal weightB = BigDecimal.valueOf(lengthB).divide(BigDecimal.valueOf(1000), 2, RoundingMode.HALF_UP);
            BigDecimal weight = BigDecimal.valueOf(length).divide(BigDecimal.valueOf(1000), 2, RoundingMode.HALF_UP);
            vo.setLengthA(BigDecimal.valueOf(lengthA));
            vo.setLengthB(BigDecimal.valueOf(lengthB));
            vo.setWeightA(weightA);
            vo.setWeightB(weightB);
            vo.setWeight(weight);
            vo.setLength(BigDecimal.valueOf(length));

            line0.setEffA(line0.getEffA().add(vo.getEffA()));
            line0.setEffB(line0.getEffB().add(vo.getEffB()));
            line0.setTimeA(line0.getTimeA().add(vo.getTimeA()));
            line0.setTimeB(line0.getTimeB().add(vo.getTimeB()));
            line0.setLengthA(line0.getLengthA().add(vo.getLengthA()));
            line0.setLengthB(line0.getLengthB().add(vo.getLengthB()));
            line0.setWeightA(line0.getWeightA().add(vo.getWeightA()));
            line0.setWeightB(line0.getWeightB().add(vo.getWeightB()));
            line0.setWeight(line0.getWeight().add(vo.getWeight()));
            line0.setLength(line0.getLength().add(vo.getLength()));
        });

        line0.setEffA(line0.getEffA().divide(BigDecimal.valueOf(8), 2, RoundingMode.HALF_UP));
        line0.setEffB(line0.getEffB().divide(BigDecimal.valueOf(8), 2, RoundingMode.HALF_UP));

        list.add(line0);
        for (int i = 1; i < 9; i++) {
            YrProdEfficiencyVO vo = arr[i];
            list.add(vo);
        }
        return list;
    }

    /**
     * 获取生产趋势
     *
     * @return 结果
     */
    @Override
    public List<YrProdTradeVO> prodTrade() {
        return mockProd();
    }

    /**
     * 获取能耗趋势
     *
     * @return 结果
     */
    @Override
    public List<YrEnergyTradeVO> energyTrade() {
        return mockEnergy();
    }

    /**
     * 获取能耗趋势
     *
     * @return 结果
     */
    public List<YrEnergyTradeVO> energyTrade(List<TwinRzCalcMonth> list) {
        List<YrEnergyTradeVO> result = new ArrayList<>();
        for (TwinRzCalcMonth cm : list) {
            YrEnergyTradeVO vo = new YrEnergyTradeVO();
            LocalDate ldt = DateUtils.toLocalDate(cm.getDataDate());
            vo.setDate(ldt.getDayOfMonth() + "");
            vo.setTips(ldt.toString());
            vo.split(cm.getLengthPrice(), cm.getDUse(), cm.getQLowUse(), cm.getSUse());
            result.add(vo);
        }
        return result;
    }

    /**
     * 获取生产趋势
     *
     * @return 结果
     */
    public List<YrProdTradeVO> prodTrade(List<TwinRzCalcMonth> list, List<TwinCalcDayYhj> yhjList) {
        Map<Date, List<TwinCalcDayYhj>> yhjMap = yhjList.stream().collect(Collectors.groupingBy(TwinCalcDayYhj::getTime));
        List<YrProdTradeVO> result = new ArrayList<>();
        for (TwinRzCalcMonth cm : list) {
            YrProdTradeVO vo = new YrProdTradeVO();
            LocalDate ldt = DateUtils.toLocalDate(cm.getDataDate());
            List<TwinCalcDayYhj> yhjs = yhjMap.get(cm.getDataDate());
            vo.setDate(ldt.getDayOfMonth() + "");
            vo.setTips(ldt.toString());
            vo.split(yhjs, cm.getLength(), cm.getWeight());
            result.add(vo);
        }
        return result;
    }

    /**
     * 模拟数据定时任务
     */
    @Override
    public void mock() {
        YrTwinVO vo = new YrTwinVO();
        YrProdLineStatusVO status = status();
        vo.setStatus(status);
        List<YrProdEfficiencyVO> effList = eff();
        int open = 0;
        for (YrProdEfficiencyVO eff : effList) {
            if (eff.getLine() == 0) {
                status.setWeight(eff.getWeight().setScale(0, RoundingMode.HALF_UP));
                status.setLength(eff.getLength().setScale(0, RoundingMode.HALF_UP));
                Random random = new Random();
                BigDecimal baiPi = eff.getLength().multiply(BigDecimal.valueOf(0.5 + random.nextDouble())).setScale(0, RoundingMode.HALF_UP);
                status.setBaiPei(baiPi);
            } else {
                if (eff.getLength().compareTo(BigDecimal.ZERO) > 0) {
                    open++;
                }
            }
        }
        status.setOpenProd(open);
        vo.setEff(effList);
        TwinRzCalcMonth calcMonth = new TwinRzCalcMonth();
        Map<String, Object> params = new HashMap<>(16);
        params.put("start", LocalDate.now().minusDays(30).toString());
        calcMonth.setParams(params);
        List<TwinRzCalcMonth> list = rzCalcMonthService.selectTwinRzCalcMonthList(calcMonth);
        TwinCalcDayYhj yhj = new TwinCalcDayYhj();
        yhj.setParams(params);
        List<TwinCalcDayYhj> yhjList = calcDayYhjService.selectTwinCalcDayYhjList(yhj);
        calcMonth.setParams(params);
        vo.setProdTrade(prodTrade(list, yhjList));
        vo.setEnergyTrade(energyTrade(list));
        redisCache.setCacheObject(CacheConstants.RZ_MOCK, vo);
    }

    private List<YrProdTradeVO> mockProd() {
        List<YrProdTradeVO> list = new ArrayList<>();
        LocalDate end = LocalDate.now().minusDays(1);
        LocalDate start = end.minusDays(30);
        do {
            YrProdTradeVO vo = new YrProdTradeVO();
            vo.setDate(start.getDayOfMonth() + "");
            vo.setTips(start.toString());
            vo.mock();
            list.add(vo);
            start = start.plusDays(1);
        } while (!start.isAfter(end));
        return list;

    }

    private List<YrEnergyTradeVO> mockEnergy() {
        List<YrEnergyTradeVO> list = new ArrayList<>();
        LocalDate end = LocalDate.now().minusDays(1);
        LocalDate start = end.minusDays(30);
        do {
            YrEnergyTradeVO vo = new YrEnergyTradeVO();
            vo.setDate(start.getDayOfMonth() + "");
            vo.setTips(start.toString());
            vo.mock();
            list.add(vo);
            start = start.plusDays(1);
        } while (!start.isAfter(end));
        return list;

    }

    /**
     * 工艺对比
     *
     * @param reqs 输入参数
     * @return 结果集
     */
    @Override
    public List<CompareVo> compare(List<YrCompareBackReq> reqs) {
        List<CompareVo> result = new ArrayList<>();
        List<TwinDeviceTypeData> all = typeService.selectTwinDeviceTypeDataAll();
        Map<Long, TwinDeviceTypeData> dataMap = all.stream()
                .collect(Collectors.toMap(
                        TwinDeviceTypeData::getDataId,
                        item -> item,
                        (existing, replacement) -> existing
                ));
        for (YrCompareBackReq req : reqs) {
            CompareVo vo = new CompareVo();
            vo.setLine(req.getLine());
            vo.setStart(req.getStart());
            vo.setEnd(req.getEnd());
            vo.setYh(10);
            CompareVo.Qz qz = vo.getQz();
            CompareVo.Hz hz = vo.getHz();
            LocalDateTime ldt = req.getEnd().minusHours(1);
            Date date = Date.from(ldt.toLocalDate().atStartOfDay(ZoneOffset.of("+8")).toInstant());
            TwinCalcHourRz search = new TwinCalcHourRz();
            search.setDataDate(date);
            search.setHour(ldt.getHour());
            search.setLine(req.getLine());
            List<TwinCalcHourRz> list = hourRzService.selectTwinCalcHourRzList(search);
            list.forEach(item -> {
                TreeMap<String, String> map = JSON.parseObject(item.getData(), TreeMap.class);
                TreeMap<String, String> treeMap = new TreeMap<>();
                treeMap.put("device", item.getDeviceName());
                map.forEach((id, value) -> {
                    TwinDeviceTypeData data = dataMap.get(Long.parseLong(id));
                    if (data != null) {
                        treeMap.put(data.getDataName(), value);
                    }
                });

                if (item.getWsName().equals("Forward")) {
                    //前整
                    switch (item.getTypeId()) {
                        case 2:
                            //四节定型机
                            if ("Y".equals(item.getOnline())) {
                                qz.setDx(qz.getDx() + 1);
                            }
                            qz.addDxPara(treeMap);
                            break;
                        case 3:
                            //双棍烫光机
                            if ("Y".equals(item.getOnline())) {
                                qz.setTg(qz.getTg() + 1);
                            }
                            qz.addTgPara(treeMap);
                            break;
                        case 4:
                            //高梳机
                            if ("Y".equals(item.getOnline())) {
                                qz.setGs(qz.getGs() + 1);
                            }
                            qz.addGsPara(treeMap);
                            break;
                        case 5:
                            //双棍刷毛
                            if ("Y".equals(item.getOnline())) {
                                qz.setSm(qz.getSm() + 1);
                            }
                            qz.addSmPara(treeMap);
                            break;
                        default:
                            break;
                    }
                } else if (item.getWsName().equals("Back")) {
                    //后整
                    switch (item.getTypeId()) {
                        case 6:
                            //六节定型机
                            if ("Y".equals(item.getOnline())) {
                                hz.setDx(hz.getDx() + 1);
                            }
                            hz.addDxPara(treeMap);
                            break;
                        case 3:
                            //双棍烫光机
                            if ("Y".equals(item.getOnline())) {
                                hz.setTg(hz.getTg() + 1);
                            }
                            hz.addTgPara(treeMap);
                            break;
                        case 4:
                            //高梳机
                            if ("Y".equals(item.getOnline())) {
                                hz.setGs(hz.getGs() + 1);
                            }
                            hz.addGsPara(treeMap);
                            break;
                        case 5:
                            //双棍刷毛
                            if ("Y".equals(item.getOnline())) {
                                hz.setSm(hz.getSm() + 1);
                            }
                            hz.addSmPara(treeMap);
                            break;
                        case 7:
                            //起毛机
                            if ("Y".equals(item.getOnline())) {
                                hz.setQm(hz.getQm() + 1);
                            }
                            hz.addQmPara(treeMap);
                            break;
                        case 9:
                            //烫剪机
                            if ("Y".equals(item.getOnline())) {
                                hz.setTj(hz.getTj() + 1);
                            }
                            hz.addTjPara(treeMap);
                            break;
                        default:
                            break;
                    }
                }
//

            });
            result.add(vo);
        }
        return result;
    }

}
