package com.jjt.rz.service.impl;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.alibaba.fastjson2.JSON;
import com.jjt.rz.domain.TwinCalcHourRz;
import com.jjt.rz.domain.TwinDeviceRz;
import com.jjt.rz.domain.TwinDeviceType;
import com.jjt.rz.mapper.TwinCalcHourRzMapper;
import com.jjt.rz.service.ITwinCalcHourRzService;
import com.jjt.rz.service.ITwinDeviceRzService;
import com.jjt.rz.service.ITwinDeviceTypeService;
import com.jjt.utils.IotService;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;

/**
 * 染整线小时工艺参数Service业务层处理
 *
 * @author wukai
 * @date 2025-06-03
 */
@Service
public class TwinCalcHourRzServiceImpl implements ITwinCalcHourRzService {
    @Resource
    private TwinCalcHourRzMapper twinCalcHourRzMapper;
    @Resource
    private IotService iotService;
    @Resource
    private ITwinDeviceRzService rzService;
    @Resource
    private ITwinDeviceTypeService typeService;
    @Resource
    private SqlSessionFactory factory;


    /**
     * 查询染整线小时工艺参数
     *
     * @param id 染整线小时工艺参数主键
     * @return 染整线小时工艺参数
     */
    @Override
    public TwinCalcHourRz selectTwinCalcHourRzById(Long id) {
        return twinCalcHourRzMapper.selectTwinCalcHourRzById(id);
    }

    /**
     * 查询染整线小时工艺参数列表
     *
     * @param twinCalcHourRz 染整线小时工艺参数
     * @return 染整线小时工艺参数
     */
    @Override
    public List<TwinCalcHourRz> selectTwinCalcHourRzList(TwinCalcHourRz twinCalcHourRz) {
        return twinCalcHourRzMapper.selectTwinCalcHourRzList(twinCalcHourRz);
    }

    /**
     * 新增染整线小时工艺参数
     *
     * @param twinCalcHourRz 染整线小时工艺参数
     * @return 结果
     */
    @Override
    public int insertTwinCalcHourRz(TwinCalcHourRz twinCalcHourRz) {
        return twinCalcHourRzMapper.insertTwinCalcHourRz(twinCalcHourRz);
    }

    /**
     * 修改染整线小时工艺参数
     *
     * @param twinCalcHourRz 染整线小时工艺参数
     * @return 结果
     */
    @Override
    public int updateTwinCalcHourRz(TwinCalcHourRz twinCalcHourRz) {
        return twinCalcHourRzMapper.updateTwinCalcHourRz(twinCalcHourRz);
    }

    /**
     * 批量删除染整线小时工艺参数
     *
     * @param ids 需要删除的染整线小时工艺参数主键
     * @return 结果
     */
    @Override
    public int deleteTwinCalcHourRzByIds(Long[] ids) {
        return twinCalcHourRzMapper.deleteTwinCalcHourRzByIds(ids);
    }

    /**
     * 删除染整线小时工艺参数信息
     *
     * @param id 染整线小时工艺参数主键
     * @return 结果
     */
    @Override
    public int deleteTwinCalcHourRzById(Long id) {
        return twinCalcHourRzMapper.deleteTwinCalcHourRzById(id);
    }

    /**
     * 按小时获取数据
     *
     * @param start 开始时间
     * @param end   结束时间
     */
    @Override
    public void hour(LocalDateTime start, LocalDateTime end) {
        Date date = Date.from(start.toLocalDate().atStartOfDay(ZoneOffset.of("+8")).toInstant());
        Long startTime = start.toInstant(ZoneOffset.of("+8")).toEpochMilli();
        Long endTime = end.toInstant(ZoneOffset.of("+8")).toEpochMilli();
        deleteTwinCalcHourRz(start.toLocalDate(), start.getHour());
        List<TwinDeviceRz> list = rzService.selectTwinDeviceRzList(new TwinDeviceRz());
        List<TwinDeviceType> typeList = typeService.selectTwinDeviceTypeAll();
        Map<Long, TwinDeviceType> typeMap = new HashMap<>(16);
        Map<String, Long> dataMap = new HashMap<>(16);
        typeList.forEach(item -> typeMap.put(item.getTypeId(), item));
        List<String> strList = new ArrayList<>();
        Map<String, TwinDeviceRz> rzMap = new HashMap<>(16);
        //查询状态结尾部分
        String statusSuffix = ".Capacity_data_1";
        list.forEach(item -> {
            String str = item.getDeviceCode() + statusSuffix;
            String mapId = item.getDevicePath() + statusSuffix;
            item.setOnline(false);
            rzMap.put(mapId, item);
            strList.add(str);

            TwinDeviceType type = typeMap.get(item.getTypeId());
            type.getTwinDeviceTypeDataList().forEach(item1 -> {
                String suffix1 = "." + item1.getDataCode();
                String mapId1 = item.getDevicePath() + suffix1;
                strList.add(item.getDeviceCode() + suffix1);
                rzMap.put(mapId1, item);
                //devicePath+suffix1当key ,dataId当value
                String key = item.getDevicePath() + suffix1;
                dataMap.put(key, item1.getDataId());
            });
        });
        String sql = "select last %s from root.tl.suxi where time>%s and time <=%s";
        sql = String.format(sql, String.join(",", strList), startTime, endTime);
        iotService.query(sql);
        JSONObject jsonObject = iotService.query(sql);
        JSONObject data = jsonObject.getJSONObject("data");
        JSONArray values = data.getJSONArray("values");
        JSONArray columnNames = data.getJSONArray("columnNames");
        JSONArray timestamps = data.getJSONArray("timestamps");
        for (int i = 0; i < values.size(); i++) {
            JSONArray da = values.getJSONArray(i);
            String type = da.getStr(0);
            String value = da.getStr(1);
            TwinDeviceRz rz = rzMap.get(type);
            if (rz != null) {
                Map<String, Object> params = rz.getParams();
                if (params == null) {
                    params = new HashMap<>();
                }
                //判断状态
                if (type.contains(statusSuffix)) {
                    if (rz.getTypeId() == 3 || rz.getTypeId() == 5 || rz.getTypeId() == 9) {
                        //双棍烫光和两棍刷毛需要按照机器状态来判断是否开机
                        //=1 设备故障停机  ，=2 人工停机 ，=3是缺布/断布停机 ，=4满布停机，=5设备上电待机中  =6设备自动运行中，=7设备过站中
                        if (value.equals("6")) {
                            rz.setOnline(true);
                        }
                    } else {
                        //其他类型只需要有值就表示开机
                        rz.setOnline(true);
                    }
                } else {
                    //工艺参数数据获取
                    String key = String.valueOf(dataMap.get(type));
                    params.put(key, value);
                    rz.setParams(params);
                }
            }
        }
        List<TwinCalcHourRz> rzList = new ArrayList<>();
        list.forEach(item -> {
            TwinCalcHourRz calc = new TwinCalcHourRz();
            calc.setDeviceId(item.getDeviceId());
            calc.setDataDate(date);
            calc.setHour(start.getHour());
            if (item.getOnline()) {
                calc.setOnline("Y");
            } else {
                calc.setOnline("N");
            }
            calc.setData(JSON.toJSONString(item.getParams()));
            rzList.add(calc);
        });

        try (SqlSession sqlSession = factory.openSession(ExecutorType.BATCH, false)) {
            if (rzList.size() > 0) {
                TwinCalcHourRzMapper mapper = sqlSession.getMapper(TwinCalcHourRzMapper.class);
                rzList.forEach(mapper::insertTwinCalcHourRz);
                sqlSession.commit();
            }
        }
    }

    /**
     * 按日期和时间删除数据
     *
     * @param date 日期
     * @param hour 时间
     */
    @Override
    public void deleteTwinCalcHourRz(LocalDate date, int hour) {
        twinCalcHourRzMapper.deleteTwinCalcHourRz(date.toString(), hour);
    }
}
