package com.jjt.rz.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

/**
 * 工艺比对视图
 *
 * @author wukai
 * @date 2025/6/3 23:47
 */
@ApiModel(value = "CompareVo", description = "工艺比对视图")
@Data
public class CompareVo {
    @ApiModelProperty("产线")
    private Integer line;
    @ApiModelProperty("开始时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime start;
    @ApiModelProperty("结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime end;
    @ApiModelProperty("前整车间")
    private Qz qz;
    @ApiModelProperty("后整车间")
    private Hz hz;
    @ApiModelProperty("印花")
    private Integer yh;
    @ApiModelProperty("电")
    private BigDecimal elect;
    @ApiModelProperty("热油")
    private BigDecimal oil;
    @ApiModelProperty("产能")
    private Integer length;

    @Data
    public static class Qz {
        @ApiModelProperty("定型次数")
        private Integer dx;
        @ApiModelProperty("定型参数")
        private List<TreeMap<String, String>> dxPara;
        @ApiModelProperty("高梳次数")
        private Integer gs;
        @ApiModelProperty("高梳参数")
        private List<TreeMap<String, String>> gsPara;
        @ApiModelProperty("双棍烫光次数")
        private Integer tg;
        @ApiModelProperty("双棍烫光参数")
        private List<TreeMap<String, String>> tgPara;
        @ApiModelProperty("双棍刷毛次数")
        private Integer sm;
        @ApiModelProperty("双棍刷毛参数")
        private List<TreeMap<String, String>> smPara;
        @ApiModelProperty("电")
        private BigDecimal elect;
        @ApiModelProperty("热油")
        private BigDecimal oil;

        public Qz() {
            this.elect = BigDecimal.ZERO;
            this.oil = BigDecimal.ZERO;
            this.dx = 0;
            this.gs = 0;
            this.tg = 0;
            this.sm = 0;
            this.dxPara = new ArrayList<>();
            this.gsPara = new ArrayList<>();
            this.tgPara = new ArrayList<>();
            this.smPara = new ArrayList<>();
        }

        public void addDxPara(TreeMap<String, String> treeMap) {
            this.dxPara.add(treeMap);
        }

        public void addGsPara(TreeMap<String, String> treeMap) {
            this.gsPara.add(treeMap);
        }

        public void addTgPara(TreeMap<String, String> treeMap) {
            this.tgPara.add(treeMap);
        }

        public void addSmPara(TreeMap<String, String> treeMap) {
            this.smPara.add(treeMap);
        }
    }

    @Data
    public static class Hz {
        @ApiModelProperty("定型次数")
        private Integer dx;
        @ApiModelProperty("定型参数")
        private List<TreeMap<String, String>> dxPara;
        @ApiModelProperty("高梳次数")
        private Integer gs;
        @ApiModelProperty("高梳参数")
        private List<TreeMap<String, String>> gsPara;
        @ApiModelProperty("双棍烫光次数")
        private Integer tg;
        @ApiModelProperty("双棍烫光参数")
        private List<TreeMap<String, String>> tgPara;
        @ApiModelProperty("双棍刷毛次数")
        private Integer sm;
        @ApiModelProperty("双棍刷毛参数")
        private List<TreeMap<String, String>> smPara;
        @ApiModelProperty("起毛次数")
        private Integer qm;
        @ApiModelProperty("起毛参数")
        private List<TreeMap<String, String>> qmPara;
        @ApiModelProperty("烫剪次数")
        private Integer tj;
        @ApiModelProperty("烫剪参数")
        private List<TreeMap<String, String>> tjPara;
        @ApiModelProperty("电")
        private BigDecimal elect;
        @ApiModelProperty("热油")
        private BigDecimal oil;

        public Hz() {
            this.elect = BigDecimal.ZERO;
            this.oil = BigDecimal.ZERO;
            this.dx = 0;
            this.gs = 0;
            this.tg = 0;
            this.sm = 0;
            this.qm = 0;
            this.tj = 0;
            this.dxPara = new ArrayList<>();
            this.gsPara = new ArrayList<>();
            this.tgPara = new ArrayList<>();
            this.smPara = new ArrayList<>();
            this.qmPara = new ArrayList<>();
            this.tjPara = new ArrayList<>();
        }

        public void addDxPara(TreeMap<String, String> treeMap) {
            this.dxPara.add(treeMap);
        }

        public void addGsPara(TreeMap<String, String> treeMap) {
            this.gsPara.add(treeMap);
        }

        public void addTgPara(TreeMap<String, String> treeMap) {
            this.tgPara.add(treeMap);
        }

        public void addSmPara(TreeMap<String, String> treeMap) {
            this.smPara.add(treeMap);
        }

        public void addQmPara(TreeMap<String, String> treeMap) {
            this.qmPara.add(treeMap);
        }

        public void addTjPara(TreeMap<String, String> treeMap) {
            this.tjPara.add(treeMap);
        }
    }

    public CompareVo() {
        this.qz = new Qz();
        this.hz = new Hz();


    }
}
